@@ -1,3 +1,26 @@
+/*
+ * The contents of this file is dual-licensed under 2 
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and 
+ * Apache License 2.0. (starting with JNA version 4.0.0).
+ * 
+ * You can freely decide which license you want to apply to 
+ * the project.
+ * 
+ * You may obtain a copy of the LGPL License at:
+ * 
+ * http://www.gnu.org/licenses/licenses.html
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "LGPL2.1".
+ * 
+ * You may obtain a copy of the Apache License at:
+ * 
+ * http://www.apache.org/licenses/
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "AL2.0".
+ */
+
 package com.sun.jna.platform.win32.COM.office;
 
 import com.sun.jna.platform.win32.Variant.VARIANT;
@@ -29,9 +52,7 @@ public class MSWord extends COMLateBindingObject {
         this.invokeNoReply("Add", getDocuments());
     }
 
-    public void openDocument(String filename, boolean bVisible)
-            throws COMException {
-        // OpenDocument
+    public void openDocument(String filename) throws COMException {
         this.invokeNoReply("Open", getDocuments(), new VARIANT(filename));
     }
 
@@ -70,7 +91,6 @@ public class MSWord extends COMLateBindingObject {
     }
 
     public Documents getDocuments() {
-        // GetDocuments
         Documents pDocuments = new Documents(this.getAutomationProperty(
                 "Documents", this.getApplication().getIDispatch()));
 
