@@ -1,14 +1,25 @@
 /* Copyright (c) 2008 Stefan Endrullis, All Rights Reserved
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * <p/>
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * The contents of this file is dual-licensed under 2 
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and 
+ * Apache License 2.0. (starting with JNA version 4.0.0).
+ * 
+ * You can freely decide which license you want to apply to 
+ * the project.
+ * 
+ * You may obtain a copy of the LGPL License at:
+ * 
+ * http://www.gnu.org/licenses/licenses.html
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "LGPL2.1".
+ * 
+ * You may obtain a copy of the Apache License at:
+ * 
+ * http://www.apache.org/licenses/
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "AL2.0".
  */
 package jnacontrib.x11.demos;
 
@@ -229,7 +240,7 @@ public class XDesktopDemo extends JFrame {
             X.Window window = windows[i];
             X.Window.Geometry geo = window.getGeometry();
             int windowId = window.getID();
-            data[i][0] = String.format("0x%08X", new Object[]{new Integer(windowId)});
+            data[i][0] = String.format("0x%08X", new Object[]{Integer.valueOf(windowId)});
             data[i][1] = "" + window.getDesktop();
             data[i][2] = window.getTitle();
             data[i][3] = "" + geo.x;
