@@ -1,18 +1,30 @@
 /* Copyright (c) 2007-2009 Timothy Wall, All Rights Reserved
+ *
+ * The contents of this file is dual-licensed under 2 
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and 
+ * Apache License 2.0. (starting with JNA version 4.0.0).
  * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * You can freely decide which license you want to apply to 
+ * the project.
  * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.  
+ * You may obtain a copy of the LGPL License at:
+ * 
+ * http://www.gnu.org/licenses/licenses.html
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "LGPL2.1".
+ * 
+ * You may obtain a copy of the Apache License at:
+ * 
+ * http://www.apache.org/licenses/
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "AL2.0".
  */
 package com.sun.jna;
 
 import java.io.File;
+import java.lang.ref.Reference;
 import java.lang.ref.WeakReference;
 import java.lang.reflect.Field;
 import java.net.MalformedURLException;
@@ -26,11 +38,11 @@ import junit.framework.TestCase;
  * that no JNI classes are directly referenced in these tests.
  */
 public class JNALoadTest extends TestCase implements Paths, GCWaits {
-    
+
     private class TestLoader extends URLClassLoader {
         public TestLoader(boolean fromJar) throws MalformedURLException {
             super(new URL[] {
-                    Platform.isWindowsCE() 
+                    Platform.isWindowsCE()
                     ? new File("/Storage Card/" + (fromJar ? "jna.jar" : "test.jar")).toURI().toURL()
                     : new File(BUILDDIR + (fromJar ? "/jna.jar" : "/classes")).toURI().toURL(),
                   }, new CloverLoader());
@@ -41,12 +53,14 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
                 assertLibraryExists();
             }
         }
-        protected Class findClass(String name) throws ClassNotFoundException {
+
+        @Override
+        protected Class<?> findClass(String name) throws ClassNotFoundException {
             String boot = System.getProperty("jna.boot.library.path");
             if (boot != null) {
                 System.setProperty("jna.boot.library.path", "");
             }
-            Class cls = super.findClass(name);
+            Class<?> cls = super.findClass(name);
             if (boot != null) {
                 System.setProperty("jna.boot.library.path", boot);
             }
@@ -60,7 +74,7 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
             throw new Error("Expected JNA jar file at " + jar + " is missing");
         }
     }
-    
+
     protected void assertLibraryExists() {
         String osPrefix = Platform.getNativeLibraryResourcePrefix();
         String name = System.mapLibraryName("jnidispatch").replace(".dylib", ".jnilib");
@@ -73,7 +87,7 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
     public void testAvoidJarUnpacking() throws Exception {
         System.setProperty("jna.nounpack", "true");
         try {
-            Class cls = Class.forName("com.sun.jna.Native", true, new TestLoader(true));
+            Class<?> cls = Class.forName("com.sun.jna.Native", true, new TestLoader(true));
 
             fail("Class com.sun.jna.Native should not be loadable if jna.nounpack=true: "
                  + cls.getClassLoader());
@@ -88,7 +102,7 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
     public void testAvoidResourcePathLoading() throws Exception {
         System.setProperty("jna.noclasspath", "true");
         try {
-            Class cls = Class.forName("com.sun.jna.Native", true, new TestLoader(false));
+            Class<?> cls = Class.forName("com.sun.jna.Native", true, new TestLoader(false));
 
             fail("Class com.sun.jna.Native should not be loadable if jna.noclasspath=true: "
                  + cls.getClassLoader());
@@ -102,7 +116,7 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
 
     public void testLoadAndUnloadFromJar() throws Exception {
         ClassLoader loader = new TestLoader(true);
-        Class cls = Class.forName("com.sun.jna.Native", true, loader);
+        Class<?> cls = Class.forName("com.sun.jna.Native", true, loader);
         assertEquals("Wrong class loader", loader, cls.getClassLoader());
         assertTrue("System property jna.loaded not set", Boolean.getBoolean("jna.loaded"));
 
@@ -113,8 +127,8 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
         assertTrue("Native library not unpacked from jar: " + path,
                    path.startsWith(System.getProperty("java.io.tmpdir")));
 
-        WeakReference ref = new WeakReference(cls);
-        WeakReference clref = new WeakReference(loader);
+        Reference<Class<?>> ref = new WeakReference<Class<?>>(cls);
+        Reference<ClassLoader> clref = new WeakReference<ClassLoader>(loader);
         loader = null;
         cls = null;
         field = null;
@@ -129,13 +143,14 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
         // Check for temporary file deletion
         File f = new File(path);
         for (int i=0;i < GC_WAITS && (f.exists() || Boolean.getBoolean("jna.loaded"));i++) {
-            Thread.sleep(GC_WAIT_INTERVAL);
+            System.gc(); // attempt to fix intermittent test failures
+            Thread.sleep(4 * GC_WAIT_INTERVAL);  // '4 *' is attempt to fix intermittent test failures
             System.gc();
         }
 
         if (f.exists()) {
-            assertTrue("Temporary jnidispatch not marked for later deletion: "
-                       + f, new File(f.getAbsolutePath()+".x").exists());
+            assertTrue("Temporary jnidispatch not marked for later deletion: " + f,
+                       new File(f.getAbsolutePath()+".x").exists());
         }
         assertFalse("System property jna.loaded not cleared", Boolean.getBoolean("jna.loaded"));
 
@@ -144,11 +159,9 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
         try {
             loader = new TestLoader(true);
             cls = Class.forName("com.sun.jna.Native", true, loader);
-        }
-        catch(Throwable t) {
+        } catch(Throwable t) {
             fail("Couldn't load class again after discarding first load: " + t.getMessage());
-        }
-        finally {
+        } finally {
             loader = null;
             cls = null;
             System.gc();
@@ -158,7 +171,7 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
     // GC Fails under OpenJDK(linux/ppc)
     public void testLoadAndUnloadFromResourcePath() throws Exception {
         ClassLoader loader = new TestLoader(false);
-        Class cls = Class.forName("com.sun.jna.Native", true, loader);
+        Class<?> cls = Class.forName("com.sun.jna.Native", true, loader);
         assertEquals("Wrong class loader", loader, cls.getClassLoader());
         assertTrue("System property jna.loaded not set", Boolean.getBoolean("jna.loaded"));
 
@@ -167,14 +180,14 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
         String path = (String)field.get(null);
         assertNotNull("Native library not found", path);
 
-        WeakReference ref = new WeakReference(cls);
-        WeakReference clref = new WeakReference(loader);
+        Reference<Class<?>> ref = new WeakReference<Class<?>>(cls);
+        Reference<ClassLoader> clref = new WeakReference<ClassLoader>(loader);
         loader = null;
         cls = null;
         field = null;
         System.gc();
         for (int i=0;i < GC_WAITS && (ref.get() != null || clref.get() != null || Boolean.getBoolean("jna.loaded"));i++) {
-            Thread.sleep(GC_WAIT_INTERVAL);
+            Thread.sleep(2 * GC_WAIT_INTERVAL);  // '2 *' is attempt to fix intermittent test failures 
             System.gc();
         }
         assertNull("Class not GC'd: " + ref.get(), ref.get());
@@ -209,9 +222,18 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
         }
     }
 
-    // Fails on Sun JVM windows (32 and 64-bit) (JVM bug)
-    // Works with IBM J9 (jdk6) windows
     public void testLoadFromUnicodePath() throws Exception {
+        if (Platform.isWindows()) {
+            String vendor = System.getProperty("java.vendor");
+            if (vendor != null) {
+                vendor = vendor.toLowerCase();
+                if (vendor.contains("oracle") || vendor.contains("sun")) {
+                    System.out.println("Skip " + getName() + " - Fails on Sun JVM windows (32 and 64-bit) (JVM bug), Works with IBM J9 (jdk6) windows");
+                    return;
+                }
+            }
+        }
+
         final String UNICODE = getName() + "-\u0444\u043b\u0441\u0432\u0443";
         File tmpdir = new File(System.getProperty("java.io.tmpdir"));
         File unicodeDir = new File(tmpdir, UNICODE);
@@ -221,7 +243,7 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
             System.setProperty("jnidispatch.preserve", "true");
             System.setProperty("jna.tmpdir", unicodeDir.getAbsolutePath());
             ClassLoader loader = new TestLoader(true);
-            Class cls = Class.forName("com.sun.jna.Native", true, loader);
+            Class<?> cls = Class.forName("com.sun.jna.Native", true, loader);
             assertEquals("Wrong class loader", loader, cls.getClassLoader());
             assertTrue("System property jna.loaded not set", Boolean.getBoolean("jna.loaded"));
 
