@@ -1,39 +1,51 @@
 /* Copyright (c) 2009 Timothy Wall, All Rights Reserved
+ *
+ * The contents of this file is dual-licensed under 2 
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and 
+ * Apache License 2.0. (starting with JNA version 4.0.0).
  * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * You can freely decide which license you want to apply to 
+ * the project.
  * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.  
+ * You may obtain a copy of the LGPL License at:
+ * 
+ * http://www.gnu.org/licenses/licenses.html
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "LGPL2.1".
+ * 
+ * You may obtain a copy of the Apache License at:
+ * 
+ * http://www.apache.org/licenses/
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "AL2.0".
  */
 package com.sun.jna;
 
-import java.util.Map;
-import java.util.HashMap;
-import java.util.Set;
-import java.util.HashSet;
 import java.lang.reflect.Method;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.List;
+import java.util.Map;
+import java.util.concurrent.TimeUnit;
 
 import junit.framework.TestCase;
 
 //@SuppressWarnings("unused")
 public class LastErrorTest extends TestCase {
-    
-    private static final Map OPTIONS = new HashMap() {{
-        put(Library.OPTION_FUNCTION_MAPPER, new FunctionMapper() {
-            public String getFunctionName(NativeLibrary library, Method m) {
-                if (m.getName().equals("noThrowLastError")
-                    || m.getName().equals("throwLastError")) {
-                    return "setLastError";
+
+    private static final Map<String, ?> OPTIONS =
+            Collections.singletonMap(Library.OPTION_FUNCTION_MAPPER, new FunctionMapper() {
+                @Override
+                public String getFunctionName(NativeLibrary library, Method m) {
+                    if (m.getName().equals("noThrowLastError")
+                        || m.getName().equals("throwLastError")) {
+                        return "setLastError";
+                    }
+                    return m.getName();
                 }
-                return m.getName();
-            }
-        });
-    }};
+            });
 
     public interface TestLibrary extends Library {
         void setLastError(int code);
@@ -42,8 +54,11 @@ public class LastErrorTest extends TestCase {
     }
 
     public static class DirectTestLibrary implements TestLibrary {
+        @Override
         public native void setLastError(int code);
+        @Override
         public native void noThrowLastError(int code);
+        @Override
         public native void throwLastError(int code) throws LastErrorException;
         static {
             Native.register(NativeLibrary.getInstance("testlib", OPTIONS));
@@ -51,16 +66,20 @@ public class LastErrorTest extends TestCase {
     }
 
     public void testLastErrorPerThreadStorage() throws Exception {
-        final TestLibrary lib = (TestLibrary)Native.loadLibrary("testlib", TestLibrary.class);
+        final TestLibrary lib = Native.loadLibrary("testlib", TestLibrary.class);
         final int NTHREADS = 100;
         final int[] errors = new int[NTHREADS];
-        Set<Thread> threads = new HashSet<Thread>();
+        List<Thread> threads = new ArrayList<Thread>(NTHREADS);
         for (int i=0;i < NTHREADS;i++) {
             final int idx = i;
-            Thread t = new Thread() { public void run() {
-                lib.setLastError(-idx-1);
-                errors[idx] = Native.getLastError();
-            }};
+            Thread t = new Thread("tLastErrorSetter-" + i) {
+                @Override
+                public void run() {
+                    lib.setLastError(-idx-1);
+                    errors[idx] = Native.getLastError();
+                }
+            };
+            t.setDaemon(true);  // so we can stop the main thread if necessary
             threads.add(t);
         }
         int EXPECTED = 42;
@@ -70,8 +89,10 @@ public class LastErrorTest extends TestCase {
             t.start();
         }
         for (Thread t : threads) {
-            t.join();
+            t.join(TimeUnit.SECONDS.toMillis(7L));
+            assertFalse("Thread " + t.getName() + " still alive", t.isAlive());
         }
+
         assertEquals("Wrong error on main thread", EXPECTED, Native.getLastError());
         for (int i=0;i < threads.size();i++) {
             assertEquals("Wrong error on thread " + i, -i-1, errors[i]);
@@ -80,15 +101,14 @@ public class LastErrorTest extends TestCase {
 
     private final int ERROR = Platform.isWindows() ? 1 : -1;
     public void testThrowLastError() {
-        TestLibrary lib = (TestLibrary)Native.loadLibrary("testlib", TestLibrary.class, OPTIONS);
+        TestLibrary lib = Native.loadLibrary("testlib", TestLibrary.class, OPTIONS);
 
         lib.noThrowLastError(ERROR);
         assertEquals("Last error not preserved", ERROR, Native.getLastError());
         try {
             lib.throwLastError(ERROR);
             fail("Method should throw LastErrorException");
-        }
-        catch(LastErrorException e) {
+        } catch(LastErrorException e) {
             assertEquals("Exception should contain error code", ERROR, e.getErrorCode());
             assertTrue("Exception should include error message: '" + e.getMessage() + "'", e.getMessage().length() > 0);
         }
@@ -102,8 +122,7 @@ public class LastErrorTest extends TestCase {
         try {
             lib.throwLastError(ERROR);
             fail("Method should throw LastErrorException");
-        }
-        catch(LastErrorException e) {
+        } catch(LastErrorException e) {
             assertEquals("Exception should contain error code", ERROR, e.getErrorCode());
             assertTrue("Exception should include error message: " + e.getMessage(), e.getMessage().length() > 0);
         }
