@@ -1,14 +1,25 @@
 /* Copyright (c) 2007-20013 Timothy Wall, All Rights Reserved
+ *
+ * The contents of this file is dual-licensed under 2 
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and 
+ * Apache License 2.0. (starting with JNA version 4.0.0).
  * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * You can freely decide which license you want to apply to 
+ * the project.
  * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.  
+ * You may obtain a copy of the LGPL License at:
+ * 
+ * http://www.gnu.org/licenses/licenses.html
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "LGPL2.1".
+ * 
+ * You may obtain a copy of the Apache License at:
+ * 
+ * http://www.apache.org/licenses/
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "AL2.0".
  */
 package com.sun.jna;
 
@@ -27,7 +38,7 @@ import java.util.Collections;
 import junit.framework.TestCase;
 
 public class LibraryLoadTest extends TestCase implements Paths {
-    
+
     private class TestLoader extends URLClassLoader {
         public TestLoader(File path) throws MalformedURLException {
             super(new URL[] { path.toURI().toURL(), },
@@ -38,7 +49,7 @@ public class LibraryLoadTest extends TestCase implements Paths {
     public void testLoadJNALibrary() {
         assertTrue("Pointer size should never be zero", Pointer.SIZE > 0);
     }
-    
+
     public void testLoadJAWT() {
         if (!Platform.HAS_AWT || !Platform.HAS_JAWT) return;
 
@@ -48,7 +59,7 @@ public class LibraryLoadTest extends TestCase implements Paths {
         // AWT is unavailable
         AWT.loadJAWT(getName());
     }
-    
+
     public void testLoadAWTAfterJNA() {
         if (!Platform.HAS_AWT) return;
 
@@ -58,7 +69,7 @@ public class LibraryLoadTest extends TestCase implements Paths {
             Toolkit.getDefaultToolkit();
         }
     }
-    
+
     public void testExtractFromResourcePath() throws Exception {
         // doesn't actually load the resource
         assertNotNull(Native.extractFromResourcePath("testlib-path", new TestLoader(new File(TESTPATH))));
@@ -115,19 +126,19 @@ public class LibraryLoadTest extends TestCase implements Paths {
     private Object load() {
         return Native.loadLibrary(Platform.C_LIBRARY_NAME, CLibrary.class);
     }
-    
+
     public void testLoadProcess() {
         Native.loadLibrary(CLibrary.class);
     }
-    
+
     public void testLoadProcessWithOptions() {
         Native.loadLibrary(CLibrary.class, Collections.EMPTY_MAP);
     }
-    
+
     public void testLoadCLibrary() {
         load();
     }
-    
+
     private void copy(File src, File dst) throws Exception {
         FileInputStream is = new FileInputStream(src);
         FileOutputStream os = new FileOutputStream(dst);
@@ -140,7 +151,7 @@ public class LibraryLoadTest extends TestCase implements Paths {
         }
         finally {
             try { is.close(); } catch(IOException e) { }
-            try { os.close(); } catch(IOException e) { } 
+            try { os.close(); } catch(IOException e) { }
         }
     }
 
@@ -163,7 +174,7 @@ public class LibraryLoadTest extends TestCase implements Paths {
             fail("Library '" + newLibName + "' at " + dst + " could not be loaded: " + e);
         }
     }
-    
+
     public void testLoadLibraryWithLongName() throws Exception {
         File tmpdir = Native.getTempDir();
         String libName = NativeLibrary.mapSharedLibraryName("testlib");
@@ -202,7 +213,7 @@ public class LibraryLoadTest extends TestCase implements Paths {
             }
         }
     }
-    
+
     public void testLoadFrameworkLibraryAbsolute() {
         if (Platform.isMac()) {
             final String PATH = "/System/Library/Frameworks/CoreServices";
@@ -255,7 +266,7 @@ public class LibraryLoadTest extends TestCase implements Paths {
     // dependent libraries in the same directory as the original
     public void testLoadDependentLibraryWithAlteredSearchPath() {
         try {
-            TestLib2 lib = (TestLib2)Native.loadLibrary("testlib2", TestLib2.class);
+            TestLib2 lib = Native.loadLibrary("testlib2", TestLib2.class);
             lib.dependentReturnFalse();
         }
         catch(UnsatisfiedLinkError e) {
@@ -271,11 +282,11 @@ public class LibraryLoadTest extends TestCase implements Paths {
     public void testLoadProperCLibraryVersion() {
         if (Platform.isWindows()) return;
 
-        CLibrary lib = (CLibrary)Native.loadLibrary("c", CLibrary.class);
+        CLibrary lib = Native.loadLibrary("c", CLibrary.class);
         assertNotNull("Couldn't get current user",
                       lib.getpwuid(lib.geteuid()));
     }
-    
+
     private static class AWT {
         public static void loadJAWT(String name) {
             Frame f = new Frame(name);
