@@ -1,18 +1,31 @@
 /* Copyright (c) 2007-2013 Timothy Wall, All Rights Reserved
+ *
+ * The contents of this file is dual-licensed under 2 
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and 
+ * Apache License 2.0. (starting with JNA version 4.0.0).
  * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * You can freely decide which license you want to apply to 
+ * the project.
  * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.  
+ * You may obtain a copy of the LGPL License at:
+ * 
+ * http://www.gnu.org/licenses/licenses.html
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "LGPL2.1".
+ * 
+ * You may obtain a copy of the Apache License at:
+ * 
+ * http://www.apache.org/licenses/
+ * 
+ * A copy is also included in the downloadable source code package
+ * containing JNA, in file "AL2.0".
  */
 package com.sun.jna;
 
 import java.io.File;
+import java.lang.reflect.Method;
+import java.nio.charset.Charset;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
@@ -24,9 +37,39 @@ import junit.framework.TestCase;
 
 //@SuppressWarnings("unused")
 public class NativeTest extends TestCase {
-    
+
     private static final String UNICODE = "[\u0444]";
 
+    public void testLoadLibraryMethods() throws Exception {
+        Class<?>[][] params = {
+                { Class.class },
+                { Class.class, Map.class },
+                { String.class, Class.class },
+                { String.class, Class.class, Map.class }
+        };
+
+        StringBuilder signature = new StringBuilder(Long.SIZE);
+        for (Class<?>[] paramTypes : params) {
+            signature.setLength(0);
+            signature.append('(');
+            for (Class<?> p : paramTypes) {
+                signature.append(Native.getSignature(p));
+            }
+            signature.append(')');
+
+            try {
+                Method m = Native.class.getMethod("loadLibrary", paramTypes);
+                Class<?> returnType = m.getReturnType();
+                signature.append(Native.getSignature(returnType));
+                assertSame("Mismatched return type for signature=" + signature, Object.class, returnType);
+//                System.out.println("===>" + m.getName() + ": " + signature);
+            } catch(NoSuchMethodError err) {
+                fail("No method for signature=" + signature);
+            }
+        }
+    }
+    
+    @SuppressWarnings("deprecation")
     public void testVersion() {
         String[] INPUTS = { "1.0", "1.0.1", "2.1.3" };
         float[] EXPECTED = { 1.0f, 1.0f, 2.1f };
@@ -55,9 +98,9 @@ public class NativeTest extends TestCase {
         // Keep stuff within the extended ASCII range so we work with more
         // limited native encodings
         String UNICODE = "Un \u00e9l\u00e9ment gr\u00e2ce \u00e0 l'index";
-        
+
         if (!UNICODE.equals(new String(UNICODE.getBytes()))) {
-            // If the extended characters aren't encodable in the default 
+            // If the extended characters aren't encodable in the default
             // encoding, punt and use straight ASCII
             UNICODE = "";
             for (char ch=1;ch < 128;ch++) {
@@ -65,23 +108,23 @@ public class NativeTest extends TestCase {
             }
         }
         final String UNICODEZ = UNICODE + "\0more stuff";
-        
+
         byte[] customEncoded = Native.getBytes(UNICODE, ENCODING);
         byte[] expected = UNICODE.getBytes(ENCODING);
         for (int i=0;i < Math.min(customEncoded.length, expected.length);i++) {
-            assertEquals("Improperly encoded (" + ENCODING + ") from Java at " + i, 
+            assertEquals("Improperly encoded (" + ENCODING + ") from Java at " + i,
                          expected[i], customEncoded[i]);
         }
-        assertEquals("Wrong number of encoded characters (" + ENCODING + ")", 
+        assertEquals("Wrong number of encoded characters (" + ENCODING + ")",
                      expected.length, customEncoded.length);
         String result = Native.toString(customEncoded, ENCODING);
-        assertEquals("Improperly decoded from native bytes (" + ENCODING + ")", 
+        assertEquals("Improperly decoded from native bytes (" + ENCODING + ")",
                      UNICODE, result);
-        
+
         assertEquals("Should truncate bytes at NUL terminator",
                      UNICODE, Native.toString(UNICODEZ.getBytes(ENCODING), ENCODING));
     }
-    
+
     public void testToStringList() {
         List<String> expected = Arrays.asList(getClass().getPackage().getName(), getClass().getSimpleName(), "testToStringList");
         StringBuilder sb = new StringBuilder();
@@ -89,7 +132,7 @@ public class NativeTest extends TestCase {
             sb.append(value).append('\0');
         }
         sb.append('\0');
-        
+
         List<String> actual = Native.toStringList(sb.toString().toCharArray());
         assertEquals("Mismatched result size", expected.size(), actual.size());
         for (int index = 0; index < expected.size(); index++) {
@@ -102,32 +145,50 @@ public class NativeTest extends TestCase {
     public void testDefaultStringEncoding() throws Exception {
         final String UNICODE = "\u0444\u043b\u0441\u0432\u0443";
         final String UNICODEZ = UNICODE + "\0more stuff";
-        byte[] utf8 = Native.getBytes(UNICODE);
+        byte[] nativeEnc = Native.getBytes(UNICODE);
         byte[] expected = UNICODE.getBytes(Native.DEFAULT_ENCODING);
-        for (int i=0;i < Math.min(utf8.length, expected.length);i++) {
-            assertEquals("Improperly encoded at " + i, 
-                         expected[i], utf8[i]);
+        for (int i=0;i < Math.min(nativeEnc.length, expected.length);i++) {
+            assertEquals("Improperly encoded at " + i,
+                         expected[i], nativeEnc[i]);
         }
-        assertEquals("Wrong number of encoded characters", expected.length, utf8.length);
-        String result = Native.toString(utf8);
-        assertEquals("Improperly decoded", UNICODE, result);
-        
+        assertEquals("Wrong number of encoded characters", expected.length, nativeEnc.length);
+        String result = Native.toString(nativeEnc);
+        // The native encoding might not support our test string; the result
+        // will then be all '?'
+        if (!result.matches("^\\?+$")) {
+            assertEquals("Improperly decoded", UNICODE, result);
+        }
+        // When the native encoding doesn't support our test string, we can only
+        // usefully compare the lengths.
         assertEquals("Should truncate bytes at NUL terminator",
-                     UNICODE, Native.toString(UNICODEZ.getBytes(Native.DEFAULT_ENCODING)));
+                UNICODE.length(), Native.toString(UNICODEZ.getBytes(Native.DEFAULT_ENCODING)).length());
     }
-    
+
     public void testCustomizeDefaultStringEncoding() {
         Properties oldprops = (Properties)System.getProperties().clone();
-        final String ENCODING = System.getProperty("file.encoding");
+        String encoding = null;
+        // Choose a charset that is not the default encoding so we can actually
+        // tell we changed it.
+        for (String charset : Charset.availableCharsets().keySet()) {
+            if (!charset.equals(Native.DEFAULT_ENCODING)) {
+                encoding = charset;
+                break;
+            }
+        }
+        assertNotNull("No available encodings other than the default!?", encoding);
         try {
-            System.setProperty("jna.encoding", ENCODING);
-            assertEquals("Default encoding should match jna.encoding setting", ENCODING, Native.getDefaultStringEncoding());
+            System.setProperty("jna.encoding", encoding);
+            assertEquals("Default encoding should match jna.encoding setting", encoding, Native.getDefaultStringEncoding());
         }
         finally {
             System.setProperties(oldprops);
         }
     }
 
+    public void testSizeof() {
+        assertEquals("Wrong bool size", 1, Native.BOOL_SIZE);
+    }
+
     public static interface TestLib extends Library {
         interface VoidCallback extends Callback {
             void callback();
@@ -137,14 +198,16 @@ public class NativeTest extends TestCase {
     public void testSynchronizedAccess() throws Exception {
         final boolean[] lockHeld = { false };
         final NativeLibrary nlib = NativeLibrary.getInstance("testlib", TestLib.class.getClassLoader());
-        final TestLib lib = (TestLib)Native.loadLibrary("testlib", TestLib.class);
-        final TestLib synchlib = (TestLib)Native.synchronizedLibrary(lib); 
+        final TestLib lib = Native.loadLibrary("testlib", TestLib.class);
+        final TestLib synchlib = (TestLib)Native.synchronizedLibrary(lib);
         final TestLib.VoidCallback cb = new TestLib.VoidCallback() {
+            @Override
             public void callback() {
                 lockHeld[0] = Thread.holdsLock(nlib);
             }
         };
         Thread t0 = new Thread() {
+            @Override
             public void run() {
                 lib.callVoidCallback(cb);
             }
@@ -155,6 +218,7 @@ public class NativeTest extends TestCase {
                    lockHeld[0]);
 
         Thread t1 = new Thread() {
+            @Override
             public void run() {
                 synchlib.callVoidCallback(cb);
             }
@@ -169,17 +233,18 @@ public class NativeTest extends TestCase {
         static class InnerTestClass extends Structure {
             interface TestCallback extends Callback { }
             static class InnerSubclass extends InnerTestClass implements Structure.ByReference { }
-            protected List getFieldOrder() { 
-                return Collections.EMPTY_LIST;
+            @Override
+            protected List<String> getFieldOrder() {
+                return Collections.<String>emptyList();
             }
         }
     }
-    
+
     public void testFindInterfaceClass() throws Exception {
-        Class interfaceClass = TestInterface.class;
-        Class cls = TestInterface.InnerTestClass.class;
-        Class subClass = TestInterface.InnerTestClass.InnerSubclass.class;
-        Class callbackClass = TestInterface.InnerTestClass.TestCallback.class;
+        Class<?> interfaceClass = TestInterface.class;
+        Class<?> cls = TestInterface.InnerTestClass.class;
+        Class<?> subClass = TestInterface.InnerTestClass.InnerSubclass.class;
+        Class<?> callbackClass = TestInterface.InnerTestClass.TestCallback.class;
         assertEquals("Enclosing interface not found for class",
                      interfaceClass, Native.findEnclosingLibraryClass(cls));
         assertEquals("Enclosing interface not found for derived class",
@@ -192,62 +257,56 @@ public class NativeTest extends TestCase {
         int TEST_ALIGNMENT = Structure.ALIGN_NONE;
         TypeMapper TEST_MAPPER = new DefaultTypeMapper();
         String TEST_ENCODING = "test-encoding";
-        Map TEST_OPTS = new HashMap() { {
-            put(OPTION_CLASSLOADER, TestInterfaceWithInstance.class.getClassLoader());
-            put(OPTION_TYPE_MAPPER, TEST_MAPPER);
-            put(OPTION_STRUCTURE_ALIGNMENT, new Integer(TEST_ALIGNMENT));
-            put(OPTION_STRING_ENCODING, TEST_ENCODING);
-        }};
-        TestInterfaceWithInstance ARBITRARY = (TestInterfaceWithInstance) 
-            Native.loadLibrary("testlib", TestInterfaceWithInstance.class, TEST_OPTS);
+        Map<String, Object> TEST_OPTS = new HashMap<String, Object>() {
+            private static final long serialVersionUID = 1L;    // we're not serializing it
+
+            {
+                put(OPTION_CLASSLOADER, TestInterfaceWithInstance.class.getClassLoader());
+                put(OPTION_TYPE_MAPPER, TEST_MAPPER);
+                put(OPTION_STRUCTURE_ALIGNMENT, Integer.valueOf(TEST_ALIGNMENT));
+                put(OPTION_STRING_ENCODING, TEST_ENCODING);
+            }
+        };
+        TestInterfaceWithInstance ARBITRARY = Native.loadLibrary("testlib", TestInterfaceWithInstance.class, TEST_OPTS);
         abstract class TestStructure extends Structure {}
     }
     public void testOptionsInferenceFromInstanceField() {
-        Class[] classes = { TestInterfaceWithInstance.class, TestInterfaceWithInstance.TestStructure.class };
+        Class<?>[] classes = { TestInterfaceWithInstance.class, TestInterfaceWithInstance.TestStructure.class };
         String[] desc = { "interface", "structure from interface" };
         for (int i=0;i < classes.length;i++) {
-            assertEquals("Wrong options found for " + desc[i]
-                         + " which provides an instance", 
-                         TestInterfaceWithInstance.TEST_OPTS,
-                         Native.getLibraryOptions(classes[i]));
-            assertEquals("Wrong type mapper found for " + desc[i], 
+            assertEquals("Wrong type mapper found for " + desc[i],
                          TestInterfaceWithInstance.TEST_MAPPER,
                          Native.getTypeMapper(classes[i]));
-            assertEquals("Wrong alignment found for " + desc[i], 
+            assertEquals("Wrong alignment found for " + desc[i],
                          TestInterfaceWithInstance.TEST_ALIGNMENT,
                          Native.getStructureAlignment(classes[i]));
-            assertEquals("Wrong string encoding found for " + desc[i], 
+            assertEquals("Wrong string encoding found for " + desc[i],
                          TestInterfaceWithInstance.TEST_ENCODING,
                          Native.getStringEncoding(classes[i]));
         }
     }
-    
+
     public interface TestInterfaceWithOptions extends Library {
         int TEST_ALIGNMENT = Structure.ALIGN_NONE;
         TypeMapper TEST_MAPPER = new DefaultTypeMapper();
         String TEST_ENCODING = "test-encoding";
-        Map OPTIONS = new HashMap() { {
-            put(OPTION_TYPE_MAPPER, TEST_MAPPER);
-            put(OPTION_STRUCTURE_ALIGNMENT, new Integer(TEST_ALIGNMENT));
-            put(OPTION_STRING_ENCODING, TEST_ENCODING);
-        }};
+        Map<String, Object> OPTIONS = new HashMap<String, Object>() {
+            private static final long serialVersionUID = 1L;    // we're not serializing it
+
+            {
+                put(OPTION_TYPE_MAPPER, TEST_MAPPER);
+                put(OPTION_STRUCTURE_ALIGNMENT, Integer.valueOf(TEST_ALIGNMENT));
+                put(OPTION_STRING_ENCODING, TEST_ENCODING);
+            }
+        };
         abstract class TestStructure extends Structure {}
     }
     public void testOptionsInferenceFromOptionsField() {
-        Class[] classes = { TestInterfaceWithOptions.class, TestInterfaceWithOptions.TestStructure.class };
-        for (int i=0;i < classes.length;i++) {
-            assertEquals("Wrong options found for interface which provides OPTIONS", 
-                         TestInterfaceWithOptions.OPTIONS,
-                         Native.getLibraryOptions(classes[i]));
-            assertEquals("Wrong type mapper found", 
-                         TestInterfaceWithOptions.TEST_MAPPER,
-                         Native.getTypeMapper(classes[i]));
-            assertEquals("Wrong alignment found", 
-                         TestInterfaceWithOptions.TEST_ALIGNMENT,
-                         Native.getStructureAlignment(classes[i]));
-            assertEquals("Wrong encoding found", 
-                         TestInterfaceWithOptions.TEST_ENCODING,
-                         Native.getStringEncoding(classes[i]));
+        Class<?>[] classes = { TestInterfaceWithOptions.class, TestInterfaceWithOptions.TestStructure.class };
+        for (Class<?> cls : classes) {
+            assertEquals("Wrong type mapper found", TestInterfaceWithOptions.TEST_MAPPER, Native.getTypeMapper(cls));
+            assertEquals("Wrong alignment found", TestInterfaceWithOptions.TEST_ALIGNMENT, Native.getStructureAlignment(cls));
+            assertEquals("Wrong encoding found", TestInterfaceWithOptions.TEST_ENCODING, Native.getStringEncoding(cls));
         }
     }
 
@@ -257,10 +316,10 @@ public class NativeTest extends TestCase {
         abstract class TestStructure extends Structure { }
     }
     public void testOptionsInferenceFromTypeMapperField() {
-        assertEquals("Wrong type mapper found for interface which provides TYPE_MAPPER", 
+        assertEquals("Wrong type mapper found for interface which provides TYPE_MAPPER",
                      TestInterfaceWithTypeMapper.TEST_MAPPER,
                      Native.getTypeMapper(TestInterfaceWithTypeMapper.class));
-        assertEquals("Wrong type mapper found for structure from interface which provides TYPE_MAPPER", 
+        assertEquals("Wrong type mapper found for structure from interface which provides TYPE_MAPPER",
                      TestInterfaceWithTypeMapper.TEST_MAPPER,
                      Native.getTypeMapper(TestInterfaceWithTypeMapper.TestStructure.class));
     }
@@ -270,10 +329,10 @@ public class NativeTest extends TestCase {
         abstract class TestStructure extends Structure { }
     }
     public void testOptionsInferenceFromAlignmentField() {
-        assertEquals("Wrong alignment found for interface which provides STRUCTURE_ALIGNMENT", 
+        assertEquals("Wrong alignment found for interface which provides STRUCTURE_ALIGNMENT",
                      Structure.ALIGN_NONE,
                      Native.getStructureAlignment(TestInterfaceWithAlignment.class));
-        assertEquals("Wrong alignment found for structure from interface which provides STRUCTURE_ALIGNMENT", 
+        assertEquals("Wrong alignment found for structure from interface which provides STRUCTURE_ALIGNMENT",
                      Structure.ALIGN_NONE,
                      Native.getStructureAlignment(TestInterfaceWithAlignment.TestStructure.class));
     }
@@ -283,14 +342,37 @@ public class NativeTest extends TestCase {
         abstract class TestStructure extends Structure { }
     }
     public void testOptionsInferenceFromEncodingField() {
-        assertEquals("Wrong encoding found for interface which provides STRING_ENCODING", 
+        assertEquals("Wrong encoding found for interface which provides STRING_ENCODING",
                      TestInterfaceWithEncoding.STRING_ENCODING,
                      Native.getStringEncoding(TestInterfaceWithEncoding.class));
-        assertEquals("Wrong encoding found for structure from interface which provides STRING_ENCODING", 
+        assertEquals("Wrong encoding found for structure from interface which provides STRING_ENCODING",
                      TestInterfaceWithEncoding.STRING_ENCODING,
                      Native.getStringEncoding(TestInterfaceWithEncoding.TestStructure.class));
     }
 
+    public interface OptionsBase extends Library {
+        int STRUCTURE_ALIGNMENT = Structure.ALIGN_NONE;
+        TypeMapper TYPE_MAPPER = new DefaultTypeMapper();
+        class TypeMappedStructure extends Structure {
+            public String stringField;
+            @Override
+            protected List <String>getFieldOrder() {
+                return Arrays.asList("stringField");
+            }
+        }
+    }
+    public interface OptionsSubclass extends OptionsBase, Library {
+        TypeMapper _MAPPER = new DefaultTypeMapper();
+        Map<String, ?> _OPTIONS = Collections.singletonMap(Library.OPTION_TYPE_MAPPER, _MAPPER);
+        OptionsSubclass INSTANCE = Native.loadLibrary("testlib", OptionsSubclass.class, _OPTIONS);
+    }
+    public void testStructureOptionsInference() {
+        Structure s = new OptionsBase.TypeMappedStructure();
+        assertEquals("Wrong structure alignment for base structure",
+                     Structure.ALIGN_NONE, Native.getStructureAlignment(s.getClass()));
+        assertEquals("Wrong type mapper for base structure", OptionsBase.TYPE_MAPPER, s.getTypeMapper());
+    }
+
     public void testCharArrayToString() {
         char[] buf = { 'a', 'b', 'c', '\0', 'd', 'e' };
         assertEquals("Wrong String generated", "abc", Native.toString(buf));
@@ -409,18 +491,18 @@ public class NativeTest extends TestCase {
                     "com.sun.jna.NativeLibraryTest",
                     "com.sun.jna.PointerTest",
                     "com.sun.jna.MemoryTest",
-                    "com.sun.jna.LibraryLoadTest", 
+                    "com.sun.jna.LibraryLoadTest",
                     "com.sun.jna.ArgumentsMarshalTest",
                     "com.sun.jna.ReturnTypesTest",
-                    "com.sun.jna.TypeMapperTest", 
+                    "com.sun.jna.TypeMapperTest",
                     "com.sun.jna.ByReferenceArgumentsTest",
-                    "com.sun.jna.LastErrorTest", 
+                    "com.sun.jna.LastErrorTest",
                     "com.sun.jna.StructureTest",// 1 wce failure (RO fields)
                     "com.sun.jna.StructureByValueTest",
                     "com.sun.jna.UnionTest",
-                    "com.sun.jna.IntegerTypeTest", 
+                    "com.sun.jna.IntegerTypeTest",
                     "com.sun.jna.VMCrashProtectionTest",
-                    "com.sun.jna.CallbacksTest", 
+                    "com.sun.jna.CallbacksTest",
                     "com.sun.jna.JNAUnloadTest",
                     "com.sun.jna.DirectTest",
                     "com.sun.jna.DirectArgumentsMarshalTest",
@@ -436,12 +518,20 @@ public class NativeTest extends TestCase {
                 System.out.println("Running tests on class " + args[i]);
                 try {
                     junit.textui.TestRunner.run((Class<? extends TestCase>) Class.forName(args[i]));
-                }
-                catch(Throwable e) {
+                } catch(Throwable e) {
                     e.printStackTrace();
                 }
             }
             try { Thread.sleep(300000); } catch(Exception e) { }
         }
     }
+    
+    public void testVersionComparison() {
+        assertTrue("Equal version", Native.isCompatibleVersion("5.1.0", "5.1.0"));
+        assertTrue("New revision", Native.isCompatibleVersion("5.2.0", "5.2.1"));
+        assertTrue("New minor provided, older minor expected", Native.isCompatibleVersion("5.1.0", "5.10.0"));
+        assertFalse("Old minor provided, new minor expected", Native.isCompatibleVersion("5.10.0", "5.1.0"));
+        assertFalse("Different major (expected < provided)", Native.isCompatibleVersion("4.0.0", "5.0.0"));
+        assertFalse("Different major (expected > provided)", Native.isCompatibleVersion("5.0.0", "4.0.0"));
+    }
 }
